//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;


texture TexPos <string uiname="Color Texture";>;
sampler SampPos = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexPos);          //apply a texture to the sampler
    MipFilter = POINT;         //sampler states
    MinFilter = POINT;
    MagFilter = POINT;
};

float2 jscale <string uiname="Julia Scale";> = { 1.0f, 1.0f };
float2 joffset <string uiname="Julia Offset";> = { 0.0f, 0.0f };

float2 cc <string uiname="Center";> = { 0.0f, 0.0f };
float2 trap <string uiname="Trap";> = { 0.5f, 0.5f };
float radius <string uiname="Radius";> = 0.0f;

float Power = 1.0f;
float Scale = 1.0f;

int iter = 64;

struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    vs2ps Out;
    Out.Pos = mul(PosO, tWVP);
    Out.TexCd = TexCd;

    return Out;
}



float4 PS_Single(vs2ps In): COLOR
{
    float2 p = -1.0 + 2.0 * In.TexCd ;
	p = p * jscale + joffset;

    float dmin = 1000000.0;
    float2 z  = p;
    bool done = false;
    
    float sum = 0;
    for( int i=0; i<iter && done ==false; i++ )
    {
        z = cc + float2( z.x*z.x - z.y*z.y, 2.0*z.x*z.y );
        float m2 = dot(z,z);
        if( m2>100.0f )
        {
            done = true;
        }
        else
        {
            float dist = distance(z,trap);
            dist -= radius;
            dmin=min(dmin,dist);
        }
    }
    float4 col;
    col.rgb = tex1D(SampPos,pow(dmin,Power)*Scale);
    col.a = 1.0;
    return col;

}





technique TJuliaSingle
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_3_0 PS_Single();
    }
}
